/**
 * C.d.L. Informatica
 * Sapienza Universita' di Roma
 * Corso di Metodologie di Programmazione
 * 
 * Prova Intermedia del 21/04/2009
 * 
 * Esercizio 4

 * Contiene solo il codice delle funzione dell'esercizio 4
 * Non ha variabili di istanza e non ha costruttori
 * 
 * @author Stefano Guerrini 
 * @version Aprile 2009
 */

import java.util.List;
import java.util.ArrayList;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.PrintWriter;

import java.util.Scanner;
import static java.lang.Integer.parseInt;

public class Esercizio4
{
    private Esercizio4() {} // nascondo il costruttore di default
    
    /**
     * Legge da un file testo una lista di studenti.
     * Ogni riga contiene i dati di uno studente nel formato
     * <matricola>:<cognome>:<nome>:<email>
     * Cattura tutte le eccezioni sollevate dai metodi di lettura
     * restituendo null in caso di errore
     * <BR/>
     * 
     * NOTA. La soluzione proposta usa la funzione split delle stringhe.
     * Altre soluzioni sono possibili usando altre funzioni di scanner.
     * Oppure separando le parti con una scansione della stringa che cerca
     * le occorrrenze del separatore ':'
     * 
     * @param nomefile      nome del file da leggere
     * @return              la lista degli studenti
     */
    static public List<Studente> leggiStudenti(String nomeFile)
    {
        Scanner scan;
        try
        {
            scan = new Scanner(new FileReader(nomeFile));
        }
        catch (FileNotFoundException e)
        {
            System.out.println("Impossibile aprire: " + nomeFile);
            return null;
        }
        scan.useDelimiter(":\n");
        List<Studente> listaStudenti = new ArrayList<Studente>();
        try
        {
            while (scan.hasNext())
            {
                String parti[] = scan.nextLine().split(":");
                if (parti.length != 4) {
                    System.err.println("Errore nel formato del file " + nomeFile);
                    scan.close();
                    return null;
                }
                Studente stud = new Studente(parseInt(parti[0]), parti[2], parti[1]);
                stud.setEmail(parti[3]);
                listaStudenti.add(stud);
            }
        }
        catch (RuntimeException e)
        {
            System.out.println("errore nel formato del file: " + nomeFile);
            scan.close();
            return null;
        }
        scan.close();
        return listaStudenti;
    }
    
    /**
     * Scrive su di un file testo una lista di studenti.
     * Ogni riga contiene i dati di uno studente nel formato
     * <matricola>:<cognome>:<nome>:<email>
     * Lascia propaagare le eccezioni sollevate, ma garantisce la corretta
     * chiusura del file in caso di errore di scrittura.
     * <BR/>
     * 
     * NOTA. La funzione println gestisce eventuali errori di scrittura
     * senza sollevare eccezioni - eventuali errori vanno verificati con
     * il metodo checkError(). Si poteva pertanto dare anche una soluzione
     * senza finally.
     * 
     * @param nomefile      nome del file da leggere
     * @return              la lista degli studenti
     */
    static public void scriviStudenti(ArrayList<Studente> listaStudenti,
                                            String nomeFile) 
                                                throws FileNotFoundException
    {
        PrintWriter file = null;
        try
        {
            file = new PrintWriter(nomeFile);
            for (Studente stud : listaStudenti)
                file.println(stud.getMatricola() + ":" 
                                + stud.getCognome() + ":"
                                + stud.getNome() + ":"
                                + stud.getEmail());
        }
        finally
        {
            if (file != null)
                file.close();
        }
    }
}
