/**
 * C.d.L. Informatica
 * Sapienza Universita' di Roma
 * Corso di Metodologie di Programmazione
 * 
 * Prova Intermedia del 21/04/2009
 * 
 * Esercizio 2
 * 
 * @author Stefano Guerrini 
 * @version Aprile 2009
 */

import java.util.List;
import java.util.ArrayList;
import java.util.ListIterator;

public class Studente extends Persona
{
    // variabili di istanza
    private final int matricola;
    private List<Integer> corsi = new ArrayList<Integer>();

    /**
     * Costruttore della classe
     * Prevede l'inizializzazione dei soli campi obbligatori
     * 
     * @param matricola matricola dello studente
     * @param nome      nome dello studente
     * @param cognome   cognome dello studente
     */
    public Studente(int matricola, String nome, String cognome)
    {
        super(nome, cognome);
        this.matricola = matricola;
    }
    
    /**
     * Costruttore della classe
     * Prevede anche l'inizializzazione del campo non
     * obbligatorio email ereditato da Persona
     * (Questo costruttore non e' strettamente necessario)
     * 
     * @param matricola matricola dello studente
     * @param nome      nome dello studente
     * @param cognome   cognome dello studente
     * @param email     email dello studente
     */
    public Studente(int matricola, String nome, String cognome, String email)
    {
        this(matricola, nome, cognome);
        setEmail(email);
    }

    /**
     * getter per la matricola dello studente
     * 
     * @return          la matricola dello studente
     */
    public int getMatricola()
    {
        return matricola;
    }
    
    /**
     * getter per i corsi dello studente
     * 
     * @return          la lista dei corsi seguiti dallo studente
     */
    private List<Integer> getCorsi()
    {
        return corsi;
    }

    /**
     * verifica se uno studente frequenta un corso
     * 
     * @param codCorso  codice del corso
     * @return          true se codCors e' nella lista dei corsi
     *                  frequentati dallo studente, false altrimenti
     */
    public boolean freqCorso(int codCorso)
    {
        for (Integer i: getCorsi())
            if (i.intValue() == codCorso)
                return true;
        return false;
    }

    /**
     * aggiunge un corso alla lista dei corsi frequentati da uno studente
     * 
     * @param codCorso  codice del corso
     * @return          true se codCors non era gia' nella lista dei corsi
     *                  frequentati dallo studente, false altrimenti
     */
    public boolean addCorso(int codCorso)
    {
        if (freqCorso(codCorso))
            return false;
        getCorsi().add(codCorso);
        return true;
    }
    
    /**
     * cancella un corso dalla lista dei corsi frequentati da uno studente
     * 
     * @param codCorso  codice del corso
     * @return          true se codCors era nella lista dei corsi
     *                  frequentati dallo studente, false altrimenti
     */
    public boolean delCorso(int codCorso)
    {
        for (ListIterator<Integer> it = getCorsi().listIterator(); it.hasNext(); )
        {
            if (it.next().intValue() == codCorso)
            {
                it.remove();
                return true;
            }
        }
        return false;
    }   

    /**
     * Sovrascrittura del metodo toString (non stampa i corsi frequentati)
     * 
     * @return      una stringa con i dati dello studente
     */
    public String toString()
    {
        return "Studente\nMatricola: " + getMatricola()
            + super.toString();
    }
}